# HMAC

This library aim to easily enable request signature handling, both for server and client side.

## Breaking changes

v0.7.0 introduce a huge change. Before v0.7.0 the lib could go deep in request validation, handling user-id and session-id, but at the cost of a low flexibility for other usage. The v0.7.0 remove all authentication specific mecanism to focus on having a simple and flexible signature mecanism. Authentication should be done in the service code or in a dedicated lib. A sub-package may come later with authentication related features such as those previously used.

## Concept

The lib is pretty basic. A Signer allow easy signature generation of a request. You can then send the signature with your request to the server so that the server can validate who sent the request, and data integrity.

Used alone, the lib won't be enough to secure your request, here are some things you can do:
- an attackant may resend a request, those replaying an action the used did (eg: in a bank system, repeate a paiment). Use a timestamp in your request and include it in your signature. The server can then ensure the timestamp of the request is newer than the last request received
- don't use your password as signing key. The password should never ever be store in clear. In addition to that, using even a derivate of the password to sign request means that if an attackant manage to get hold of the key, it can perform any action it want. A better way is to use a session system. Eg:
	- use a derivate of the password for authentication, the request return a session id and key to use for next requests
	- add an expiration date to the session, after which it is not valid anymore. That way, event if the session key leak, the attackant will quickly be limited
	- using session, if a session key is compromised the user can revoke it to prevent any further use by the attackant

## How to use

### Server

The current version of the lib only contains a middleware for [echo](echo.labstack.com/) router. See middleware/echo/readme.md for more information on how to use it

### Client

In the client folder you will find a custom `http.RoundTripper` called `SignerRoundTripper`.

`SignerRoundTripper` as its name may let you know, is used to Signe request of a client automatically. A typical usage would to be to create an `http.Client` associated with a user session using that RoundTripper, so that every request will be signed.


```go
signer := hmac.NewSigner(
			hmac.SignerWithHashFunction(sha256.New),
			hmac.SignerWithRequestEncoder(templateEncoder),
			hmac.SignerWithSignatureEncoder(
				func(b []byte) ([]byte, error) {
					return []byte(base64.StdEncoding.EncodeToString(b)), nil
				},
			),
		)

signerClient := http.Client{}
signerClient.Transport = client.NewSignerRoundTripper(
	http.DefaultTransport,
	signer,
	[]byte("dummyKey"),
	"X-Signature",
)
```