# echo middleware

This folder contains middleware for [echo](echo.labstack.com/) router

This middleware only ensure signature is legit. Alone, the signature is not enought, here are a few recommandation:
- your user should include a timestamp in the signed data, that way you can prevent message re-play
- you should not use the user password as signing key, except on loggin. When the user login, provide the user a session ID and key, with a peremption date. Following request should use that session key. Doing so:
    - The user can logout by invalidating the session
    - If the session happen to be compromised, revoking the session will prevent any unwanted action from any third party
    - Changing the session key frequently reduce risk, if a third-party manage to get the key, it will only be valid for a limit period of time


Here is an example usage:

```go
import hmacMiddleware "gitlab.bad-wolf.fr/wolf/hmac/middleware/echo"

ksm := func(c echo.Context) (key []byte, signature []byte, err error) {
    signatureB64 := c.Request().Header.Get("X-Signature")
    return []byte("dummyKey"), []byte(signatureB64), err
}

signer := hmac.NewSigner(
    hmac.SignerWithHashFunction(sha256.New),
    hmac.SignerWithRequestEncoder(templateEncoder),
    hmac.SignerWithSignatureEncoder(
        func(b []byte) ([]byte, error) {
            return []byte(base64.StdEncoding.EncodeToString(b)), nil
        },
    ),
)

e := echo.New()
auth := e.Group("/auth", hmacMiddleware.SignatureChecker(signer, ksm, true))
```

The `ksm` var contains a function used by the `hmacMiddleware.SignatureChecker` to retrieve the signature from a request and the key associated to the said request. Here the signature is in the `X-Signature` header. The key is static, but may for example be linked to a user-id from header.

In this example `hmacMiddleware.SignatureChecker(signer, ksm, true)` the `signatureRequired` is set to false, which means that an unauthenticated user may perform requests. Your handler can use the `hmacMiddleware.SignatureOK(c)` function to know if the calling user is authenticated or not. Furthermore, you can, use `hmacMiddleware.SignatureEnforce()` to enforce the signature validation on specific route of a group, event thought the `signatureRequired` was set to false.